import path from "path";
import fs from "fs";
import { readJson, runCommands } from "node-pearls";

export default async function(options){
    var currentDir = process.cwd();
    if(options.dir){
        currentDir = path.join(currentDir, options.dir);
    }

    const packageJsonFile = path.join(currentDir, "package.json");
    const packageJson = readJson.sync(packageJsonFile);
    packageJson.version = options.version || (function(versions){
        versions[versions.length - 1] = +versions[versions.length - 1] + 1;
        return versions.join(".");
    })(packageJson.version.split("."));
    fs.writeFileSync(packageJsonFile, JSON.stringify(packageJson, null, "   "));

    const tag = options.dir ? `${packageJson.name}@${packageJson.version}` : packageJson.version;

    await runCommands(`
        npm publish --registry https://registry.npmjs.org/
        git add .
        git commit -m "${tag}"
        git push
    `, {
        cwd: currentDir
    });

    if(!options.dir){
        await runCommands(`
            git tag -a ${tag} -m '${tag}'
            git push origin ${tag}
        `, {
            cwd: currentDir
        });
    }
}