import fs from "fs";
import path from "path";
import http from "http";
import glob from "glob";
import utils from "node-pearls";
import config from "../config";

const loader = "enjoy-loader@0.1/loader.js";
const booter = "enjoy-loader@0.1/booter.js";

export default function(options){
    const root = process.cwd();
    const enjoyJson = utils.readJson.sync(path.join(root, "enjoy.json"));
    const packageJson = utils.readJson.sync(path.join(root, "package.json"));
    const packageName = [packageJson.name, packageJson.version].join("@");
    const entryRules = enjoyJson.entries["default"].entries;
    var entries = [];
    Promise.all(entryRules.map(function(entry){
        return new Promise(function(resolve, reject){
            glob(entry, {
                cwd: root
            }, function(err, files){
                entries = entries.concat(files.map(function(file){
                    return path.join(packageName, file);
                }));
                resolve();
            });
        });
    })).then(function(){
        const publishTemp = path.join(process.env.HOME || __dirname, ".publish-temp");
        console.log(entries);
        getAbbrs([loader].concat(entries), options.env);
    });
}

var backups = [];

function getAbbrs(entries, env) {
    var files = [];
    var abbrHash = {};

    Promise.all(entries.map(function (entry) {
        return new Promise(function (resolve, reject) {
            var url = config.server[env].deps + entry;
            // 获取依赖信息
            http.get(url, function (response) {
                parseDepsResponse(response, function (abbrs) {
                    abbrHash[entry] = abbrs;
                    resolve();
                }, function (err) {
                    reject(new Error(`
                        url: ${url}
                        http code: ${err.code}
                        error message: ${err.message}
                    `));
                });
            });
        });
    })).then(async function () {
        const booterCode = await getBooterCode();
        
        var _entries = [
            loader
        ].concat(entries[0]).map(function (entry) {
            return `$atom.script("${entry}");`;
        }).join("");

        console.log(`\n\n\n\n\n\n\n\n\n\n--------------------* 页面脚本 *--------------------`);
        var script = `<script>${booterCode}$atom.$map=${JSON.stringify(abbrHash)};$atom.$backup=${JSON.stringify(backups)};$atom.config=${JSON.stringify(config.server[env].config)};${_entries}</script>`;
        console.log(script);
    }).catch(function (err) {
        console.log(err);
    });
}

// 解析response数据
function parseResponse(response, resolve, reject) {
    var data = "";
    if (response.statusCode >= 200 && response.statusCode < 300) {
        response.setEncoding('utf8');
        response.on('data', function (chunk) {
            data += chunk;
        });
        response.on('end', function () {
            resolve(data);
        });
    } else {
        reject({
            code: response.statusCode,
            message: response.statusMessage
        });
    }
}

function parseDepsResponse(response, resolve, reject){
    parseResponse(response, function(data){
        try {
            data = JSON.parse(data);
            resolve(data.files.join(","));
        } catch (e) {
            console.error(e);
            console.log(data);
            reject({
                code: 500,
                message: "json解析错误"
            });
        }
    }, reject);
}

function getBooterCode(){
    return new Promise(function(resolve, reject){
        var url = config.server[env].deps + booter;
        // 获取依赖信息
        http.get(url, function (response) {
            parseDepsResponse(response, function (abbrs) {
                var url = config.server[env].config.base + abbrs;
                http.get(url, function(response){
                    parseResponse(response, function(code){
                        resolve(code);
                    }, function(err){
                        reject(new Error(`
                            url: ${url}
                            http code: ${err.code}
                            error message: ${err.message}
                        `));
                    });
                });
            }, function (err) {
                reject(new Error(`
                    url: ${url}
                    http code: ${err.code}
                    error message: ${err.message}
                `));
            });
        });
    });
}