import config from "../../config";
import path from "path";
import fs from "fs";
import { runCommands } from "node-pearls";
import console from "cli-console";
import rnBusiness from "./publish-rn-business";
import webBusiness from "./publish-web-business";

export default async function(options){
    console.log("=====index");
    // 要打包的平台
    const platforms = options.platforms ? options.platforms.split(",").map(platform => platform.trim()) : [];
    const businessName = options.businessName;
    const iosZipname = options.iosZipname || businessName + "_package";
    const androidZipname = options.androidZipname || businessName + "_package";
    const pushBranch = options.increment ? "increment" : (options.pushBranch || "master");
    const version = options.version;
    const buildId = options.buildId;
    const currentDir = process.cwd();

    var depPlatforms = [];
    var hasRn = platforms.some(platform => ["ios", "android"].indexOf(platform) !== -1);
    if(hasRn){
        depPlatforms = ["rn"].concat(platforms.filter(platform => ["ios", "android"].indexOf(platform) === -1));
    }else{
        depPlatforms = platforms;
    }

    // 安装依赖
    console.title("安装依赖");
    await runCommands(`npm i`);
    await runCommands(depPlatforms.map(platform => `enjoy update --${platform === "rn" ? "ios" : platform}`).join("\n"));

    try{
        // 锁定cli版本
        await runCommands("npm shrinkwrap", {
            cwd: currentDir
        });
        fs.renameSync(
            path.join(currentDir, "npm-shrinkwrap.json"),
            path.join(currentDir, "_npm-shrinkwrap.json")
        );
        // 锁定support版本
        await Promise.all(depPlatforms.map(platform => (async () => {
            let dir = platform === "rn" ? path.join(currentDir, "rn") : path.join(currentDir, config["group-hash"][platform], platform);
            await runCommands("npm shrinkwrap", {
                cwd: dir
            });
            fs.renameSync(
                path.join(dir, "npm-shrinkwrap.json"),
                path.join(dir, "_npm-shrinkwrap.json")
            );
        })()));
    }catch(e){
        console.log(e);
    }

    // 打包
    console.title("打包");
    await runCommands(platforms.map(platform => {
        let zipname = {
            "ios": iosZipname,
            "android": androidZipname
        }[platform] || "";

        let argus = [];

        if(zipname){
            argus.push(`--zipname ${zipname}`);
        }else if(options.combo){
            argus.push("--combo");
        }else if(!options.combo){
            if(options.publishVersion){
                argus.push("--publishVersion "+ options.publishVersion);
            }
            argus.push("--incremental");
        }

        if(options.release){
            argus.push("--release");
        }

        return [`enjoy bundle --${platform}`].concat(argus).join(" ");
    }).join("\n"))

    //部署或者其他处理
    if(options.platforms&&(options.platforms.indexOf("ios") > -1 || options.platforms.indexOf("android") > -1)){
        rnBusiness(Object.assign(options,{
            "platforms": options.platforms,
            "iosZipname": iosZipname,
            "androidZipname": androidZipname,
            "pushBranch": pushBranch
        }))
    }else if(options.platforms){
        webBusiness(Object.assign(options,{
            "platforms": options.platforms
        }))
    }

    console.title("提交git");
    // await runCommands(`
    //     git add .
    //     git commit -m "publish@${version}"
    //     git push
    // `, {
    //     cwd: currentDir
    // });
}