import config from "../config";
import rimraf from "rimraf";
import path from "path";
import fs from "fs";
import { copyFiles, readJson, mkdirs, runCommands } from "node-pearls";
import console from "cli-console";
import archiver from "archiver";

// 将js文件先打tar包再打zip包
function tarzip(file, outputFile, zipname){
    return new Promise(function(resolve, reject){
        var dir = path.dirname(file);
        
        var tarfile = path.join(dir, zipname + ".tar");

        var input = fs.createReadStream(file);
        var tarwrite = fs.createWriteStream(tarfile);
        // 打tar包
        let tar = archiver('tar');
        tar.on('error', function(err){
            reject(err);
        });
        tar.append(input, {
            name: "framework.js"
        });
        tar.pipe(tarwrite);
        tar.finalize();
        tarwrite.on("close", function(){
            var tarread = fs.createReadStream(tarfile);
            var output = fs.createWriteStream(outputFile);
            // 打zip包
            let zip = archiver('zip');
            zip.on('error', function(err){
                reject(err);
            });

            zip.append(tarread, {
                name: zipname + ".tar"
            });
            zip.pipe(output);

            zip.finalize();
            
            output.on("close", function(){
                fs.unlink(tarfile);
                resolve();
            });
        });
    });
}

export default async function(options){
    // 要打包的平台
    const platforms = options.platforms ? options.platforms.split(",").map(platform => platform.trim()) : [];
    const zipname = options.zipname || "framework";
    const iosZipname = options.iosZipname || zipname + "_package";
    const androidZipname = options.androidZipname || zipname + "_package";
    const pushBranch = options.pushBranch || "master";
    const versionRule = options.version;
    const currentDir = process.cwd();
    const tmpInstallDir = path.join(config["tmp-dir"], "framework-tmp-dir");

    mkdirs.sync(tmpInstallDir);
    fs.writeFileSync(path.join(tmpInstallDir, "package.json"), JSON.stringify({
        name: "framework-tmp-dir",
        version: "0.0.1"
    }));
    await Promise.all(platforms.map(platform => (async () => {
        let modName = `enjoy-bundle-framework-${platform}`;
        await runCommands(`npm i ${modName}` + (versionRule ? `@${versionRule}` : ""), {
            cwd: tmpInstallDir
        });
        let version = readJson.sync(path.join(tmpInstallDir, "node_modules", modName, "package.json")).version;

        if(platform === "ios"){
            let gitname = `elongreactnativepackages@${pushBranch}`;
            let gitdir = path.join(config["tmp-dir"], gitname);

            console.title("拉取项目");
            if(!fs.existsSync(gitdir)){
                await runCommands(`
                    git clone git@git.17usoft.com:ios/elongreactnativepackages.git ${gitdir} -b ${pushBranch} --depth 1
                `);
            }else{
                await runCommands(`
                    git pull
                `, {
                    cwd: gitdir
                });
            }

            console.title("压缩拷贝文件");
            await tarzip(
                path.join(path.join(tmpInstallDir, "node_modules", modName, options.release ? "index.js" : "index.dev.js")),
                path.join(gitdir, iosZipname + ".zip"),
                iosZipname
            );

            console.title("提交git");
            await runCommands(`
                git add .
                git commit -m 0:framework@${version}:true
                git push
            `, {
                cwd: gitdir
            });
        }else if(platform === "android"){
            let gitname = `as_elongrnplugin_44_jsbundle@${pushBranch}`;
            let gitdir = path.join(config["tmp-dir"], gitname);

            console.title("拉取项目");
            if(!fs.existsSync(gitdir)){
                await runCommands(`
                    git clone git@git.17usoft.com:elongtravel_android/as_elongrnplugin_44_jsbundle.git ${gitdir} -b ${pushBranch} --depth 1
                `);
            }else{
                await runCommands(`
                    git pull
                `, {
                    cwd: gitdir
                });
            }

            console.title("压缩拷贝文件");
            await tarzip(
                path.join(path.join(tmpInstallDir, "node_modules", modName, options.release ? "index.js" : "index.dev.js")),
                path.join(gitdir, androidZipname + ".zip"),
                androidZipname
            );

            console.title("提交git");
            await runCommands(`
                git add .
                git commit -m 0:framework@${version}:true
                git push
            `, {
                cwd: gitdir
            });
        }else{

        }
    })()));
}