"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _config = require("../config");

var _config2 = _interopRequireDefault(_config);

var _rimraf = require("rimraf");

var _rimraf2 = _interopRequireDefault(_rimraf);

var _child_process = require("child_process");

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _nodePearls = require("node-pearls");

var _cliConsole = require("cli-console");

var _cliConsole2 = _interopRequireDefault(_cliConsole);

var _branch = require("../gitlab/branch");

var _branch2 = _interopRequireDefault(_branch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * 将版本规则解析为固定的打包版本号
 * @param {string} versionRule - 版本规则
 * @param {string} packageVersion - 当前安装项目的版本
 * @description
 		目前支持的版本规则如下（参见语义化版本semver）：
 			1.2.3			-	指定版本，模块打包后为 name/1.2.3/...
 			波浪线~			-	自动升级版本，例如 ~1.2.3 模块打包后为 name/1.2/...
 			补注号^(默认)		-	自动升级版本，例如 ^1.2.3 模块打包后为 name/1/...
 			x范围			-	自动升级版本，例如 1.2.x 模块打包后为 name/1.2/...
 			其他规则			-	转换为当前安装项目的版本，不自动升级
 */
function parseVersion(versionRule) {
    var originalVersionRule = versionRule;
    versionRule = versionRule.split("-")[0];

    if (/^\d+\.\d+\.\d+$/.test(versionRule)) {
        return originalVersionRule;
    } else if (/^\~\d+(\.\d+){0,2}$/.test(versionRule)) {
        return versionRule.replace(/^\~/, "").split(".").slice(0, 2).join(".");
    } else if (/^\^\d+(\.\d+){0,2}$/.test(versionRule)) {
        versionRule = versionRule.replace(/^\^/, "").split(".");
        return versionRule.slice(0, versionRule.findIndex(item => item != 0) + 1).join(".");
    } else if (/^\d+(\.\d+)?\.x$/.test(versionRule)) {
        return versionRule.replace(/\.x$/, "");
    } else {
        _cliConsole2.default.warn(`版本规则${versionRule}暂时不支持`);
        return "";
    }
}

exports.default = (() => {
    var _ref = _asyncToGenerator(function* (options) {
        // 
        const platforms = options.platforms ? options.platforms.split(",").map(function (platform) {
            return platform.trim();
        }) : [];
        // 临时enjoy项目名
        const tempProjectName = "FrameworkBundle";
        // 创建的临时项目地址
        const tempProjectPath = _path2.default.join(_config2.default["tmp-dir"], tempProjectName);

        // 删除原有的项目
        _rimraf2.default.sync(tempProjectPath);
        // 创建指定版本的临时项目
        var initCommandArgus = [];
        if (options.version && typeof options.version === "string") {
            initCommandArgus.push(`--version ${options.version}`);
        }
        var depPlatforms = [];
        var hasRn = platforms.some(function (platform) {
            return ["ios", "android"].indexOf(platform) !== -1;
        });
        if (hasRn) {
            depPlatforms = ["rn"].concat(platforms.filter(function (platform) {
                return ["ios", "android"].indexOf(platform) === -1;
            }));
        } else {
            depPlatforms = platforms;
        }
        initCommandArgus.push(`--platforms ${depPlatforms.join(",")}`);

        _nodePearls.mkdirs.sync(_config2.default["tmp-dir"]);
        try {
            yield (0, _nodePearls.runCommands)([`enjoy init ${tempProjectName} --logs --pure`].concat(initCommandArgus).join(" "), {
                cwd: _config2.default["tmp-dir"]
            });
        } catch (e) {
            _cliConsole2.default.log(e);
        }
        try {
            // 锁定cli版本
            yield (0, _nodePearls.runCommands)("npm shrinkwrap", {
                cwd: tempProjectPath
            });
        } catch (e) {
            _cliConsole2.default.log(e);
        }
        try {
            // 锁定support版本
            yield Promise.all(depPlatforms.map(function (platform) {
                return _asyncToGenerator(function* () {
                    yield (0, _nodePearls.runCommands)("npm shrinkwrap", {
                        cwd: platform === "rn" ? _path2.default.join(tempProjectPath, "rn") : _path2.default.join(tempProjectPath, _config2.default["group-hash"][platform], platform)
                    });
                })();
            }));
        } catch (e) {
            _cliConsole2.default.log(e);
        }
        // 打包各个平台
        yield Promise.all(platforms.map(function (platform) {
            return _asyncToGenerator(function* () {
                let group = _config2.default["group-hash"][platform];
                let platformDir = _path2.default.join(tempProjectPath, group, platform);
                let saveGitName = "framework-bundle";
                let saveGitPath = _path2.default.join(platformDir, saveGitName);
                let nodeModulesDir = group === "rn" ? _path2.default.join(tempProjectPath, group) : platformDir;
                let supportModuleName = group === "rn" ? "enjoy-rn-support-elong" : platform === "h5" ? "enjoy-web-support-elong" : `enjoy-${platform}-support-elong`;
                let platformVersion = parseVersion(_nodePearls.readJson.sync(_path2.default.join(nodeModulesDir, "package.json")).dependencies[supportModuleName]);
                let branchName = `${platform}@${platformVersion}`;
                // 检查项目的指定分支是否存在，如果不存在则创建此分支
                if (!(yield _branch2.default.exists(saveGitName, branchName))) {
                    yield _branch2.default.create(saveGitName, branchName);
                }
                // 克隆存储git仓库指定平台分支
                _rimraf2.default.sync(saveGitPath);
                yield (0, _nodePearls.runCommands)(`git clone git@git.17usoft.com:enjoy/framework-bundle.git ${saveGitPath} -b ${branchName} --depth 1`);
                // 拷贝锁定版本
                try {
                    yield (0, _nodePearls.copyFiles)([{
                        src: _path2.default.join(tempProjectPath, "npm-shrinkwrap.json"),
                        dist: _path2.default.join(saveGitPath, "npm-shrinkwrap.cli.json")
                    }, {
                        src: _path2.default.join(nodeModulesDir, "npm-shrinkwrap.json"),
                        dist: _path2.default.join(saveGitPath, "npm-shrinkwrap.support.json")
                    }]);
                } catch (e) {
                    _cliConsole2.default.log(e);
                }

                if (group === "rn") {
                    let bundleDir = _path2.default.join(platformDir, "bundle", "combo");
                    _cliConsole2.default.title(`打${platform}平台dev包`);
                    // 打包dev
                    yield (0, _nodePearls.runCommands)(`enjoy bundle --framework --${platform}`, {
                        cwd: tempProjectPath
                    });
                    _cliConsole2.default.title(`拷贝${platform}平台dev包`);
                    // 拷贝打包结果
                    yield (0, _nodePearls.copyFiles)(["framework.js", "framework.debug.js"].map(function (file) {
                        return {
                            src: _path2.default.join(bundleDir, file),
                            dist: _path2.default.join(saveGitPath, file.replace(/framework/, "index.dev"))
                        };
                    }));
                    _cliConsole2.default.title(`打${platform}平台release包`);
                    // 打包release
                    yield (0, _nodePearls.runCommands)(`enjoy bundle --framework --${platform} --release`, {
                        cwd: tempProjectPath
                    });
                    _cliConsole2.default.title(`拷贝${platform}平台release包`);
                    // 拷贝打包结果
                    yield (0, _nodePearls.copyFiles)(["framework.js", "framework.js.map", "framework.debug.js", "framework.debug.js.map"].map(function (file) {
                        return {
                            src: _path2.default.join(bundleDir, file),
                            dist: _path2.default.join(saveGitPath, file.replace(/framework/, "index"))
                        };
                    }));
                } else {
                    let bundleDir = _path2.default.join(platformDir, "bundle");
                    _cliConsole2.default.title("打dev包");
                    // 打dev包
                    yield (0, _nodePearls.runCommands)(`enjoy bundle --framework --${platform}`, {
                        cwd: tempProjectPath
                    });
                    _cliConsole2.default.title("拷贝dev包");
                    yield (0, _nodePearls.copyFiles)(["framework.zip"
                    // ,"framework.files"
                    ].map(function (file) {
                        return {
                            src: _path2.default.join(bundleDir, file),
                            dist: _path2.default.join(saveGitPath, file.replace(/framework/, "index.dev"))
                        };
                    }));
                    _cliConsole2.default.title("打dev combo包");
                    // 打dev combo包
                    yield (0, _nodePearls.runCommands)(`enjoy bundle --framework --${platform} --combo`, {
                        cwd: tempProjectPath
                    });
                    _cliConsole2.default.title("拷贝dev combo包");
                    yield (0, _nodePearls.copyFiles)(["framework.js"].map(function (file) {
                        return {
                            src: _path2.default.join(bundleDir, "combo", file),
                            dist: _path2.default.join(saveGitPath, file.replace(/framework/, "index.dev"))
                        };
                    }));
                    _cliConsole2.default.title("打release包");
                    // 打release包
                    yield (0, _nodePearls.runCommands)(`enjoy bundle --framework --${platform} --release`, {
                        cwd: tempProjectPath
                    });
                    _cliConsole2.default.title("拷贝release包");
                    yield (0, _nodePearls.copyFiles)(["framework.zip"
                    // ,"framework.files"
                    ].map(function (file) {
                        return {
                            src: _path2.default.join(bundleDir, file),
                            dist: _path2.default.join(saveGitPath, file.replace(/framework/, "index"))
                        };
                    }));
                    _cliConsole2.default.title("打release combo包");
                    // 打release combo包
                    yield (0, _nodePearls.runCommands)(`enjoy bundle --framework --${platform} --release --combo`, {
                        cwd: tempProjectPath
                    });
                    _cliConsole2.default.title("拷贝release combo包");
                    yield (0, _nodePearls.copyFiles)(["framework.js", "framework.js.map"].map(function (file) {
                        return {
                            src: _path2.default.join(bundleDir, "combo", file),
                            dist: _path2.default.join(saveGitPath, file.replace(/framework/, "index"))
                        };
                    }));
                }

                _cliConsole2.default.title("升级版本号");
                // 更新版本号
                const version = function () {
                    let packageJsonFile = _path2.default.join(saveGitPath, "package.json");
                    let packageJson = _nodePearls.readJson.sync(packageJsonFile);
                    let modName = `enjoy-bundle-framework-${platform}`;
                    if (packageJson.name !== modName) {
                        packageJson.name = modName;
                        packageJson.version = function (versions) {
                            for (let i = 0; i < 3; i++) {
                                if (!versions[i]) {
                                    versions[i] = "0";
                                }
                            }
                            return versions.join(".");
                        }(platformVersion.split("."));
                    } else {
                        let version = packageJson.version.split(".");
                        version[version.length - 1] = +version[version.length - 1] + 1;
                        version = version.join(".");
                        packageJson.version = version;
                    }
                    _fs2.default.writeFileSync(packageJsonFile, JSON.stringify(packageJson, null, "   "));
                    return packageJson.version;
                }();

                _cliConsole2.default.title("发布npm");
                yield (0, _nodePearls.runCommands)(`npm publish --registry https://registry.npmjs.org/`, {
                    cwd: saveGitPath
                });
                //
                _cliConsole2.default.title("提交git");
                yield (0, _nodePearls.runCommands)(`
            git add .
            git commit -m "${version}"
            git push
        `, {
                    cwd: saveGitPath
                });
            })();
        }));

        _cliConsole2.default.success("打包完成");
    });

    return function (_x) {
        return _ref.apply(this, arguments);
    };
})();