"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (options) {
    const root = process.cwd();
    const enjoyJson = _nodePearls2.default.readJson.sync(_path2.default.join(root, "enjoy.json"));
    const packageJson = _nodePearls2.default.readJson.sync(_path2.default.join(root, "package.json"));
    const packageName = [packageJson.name, packageJson.version].join("@");
    const entryRules = enjoyJson.entries["default"].entries;
    var entries = [];
    Promise.all(entryRules.map(function (entry) {
        return new Promise(function (resolve, reject) {
            (0, _glob2.default)(entry, {
                cwd: root
            }, function (err, files) {
                entries = entries.concat(files.map(function (file) {
                    return _path2.default.join(packageName, file);
                }));
                resolve();
            });
        });
    })).then(function () {
        const publishTemp = _path2.default.join(process.env.HOME || __dirname, ".publish-temp");
        console.log(entries);
        getAbbrs([loader].concat(entries), options.env);
    });
};

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _http = require("http");

var _http2 = _interopRequireDefault(_http);

var _glob = require("glob");

var _glob2 = _interopRequireDefault(_glob);

var _nodePearls = require("node-pearls");

var _nodePearls2 = _interopRequireDefault(_nodePearls);

var _config = require("../config");

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const loader = "enjoy-loader@0.1/loader.js";
const booter = "enjoy-loader@0.1/booter.js";

var backups = [];

function getAbbrs(entries, env) {
    var files = [];
    var abbrHash = {};

    Promise.all(entries.map(function (entry) {
        return new Promise(function (resolve, reject) {
            var url = _config2.default.server[env].deps + entry;
            // 获取依赖信息
            _http2.default.get(url, function (response) {
                parseDepsResponse(response, function (abbrs) {
                    abbrHash[entry] = abbrs;
                    resolve();
                }, function (err) {
                    reject(new Error(`
                        url: ${url}
                        http code: ${err.code}
                        error message: ${err.message}
                    `));
                });
            });
        });
    })).then(_asyncToGenerator(function* () {
        const booterCode = yield getBooterCode();

        var _entries = [loader].concat(entries[0]).map(function (entry) {
            return `$atom.script("${entry}");`;
        }).join("");

        console.log(`\n\n\n\n\n\n\n\n\n\n--------------------* 页面脚本 *--------------------`);
        var script = `<script>${booterCode}$atom.$map=${JSON.stringify(abbrHash)};$atom.$backup=${JSON.stringify(backups)};$atom.config=${JSON.stringify(_config2.default.server[env].config)};${_entries}</script>`;
        console.log(script);
    })).catch(function (err) {
        console.log(err);
    });
}

// 解析response数据
function parseResponse(response, resolve, reject) {
    var data = "";
    if (response.statusCode >= 200 && response.statusCode < 300) {
        response.setEncoding('utf8');
        response.on('data', function (chunk) {
            data += chunk;
        });
        response.on('end', function () {
            resolve(data);
        });
    } else {
        reject({
            code: response.statusCode,
            message: response.statusMessage
        });
    }
}

function parseDepsResponse(response, resolve, reject) {
    parseResponse(response, function (data) {
        try {
            data = JSON.parse(data);
            resolve(data.files.join(","));
        } catch (e) {
            console.error(e);
            console.log(data);
            reject({
                code: 500,
                message: "json解析错误"
            });
        }
    }, reject);
}

function getBooterCode() {
    return new Promise(function (resolve, reject) {
        var url = _config2.default.server[env].deps + booter;
        // 获取依赖信息
        _http2.default.get(url, function (response) {
            parseDepsResponse(response, function (abbrs) {
                var url = _config2.default.server[env].config.base + abbrs;
                _http2.default.get(url, function (response) {
                    parseResponse(response, function (code) {
                        resolve(code);
                    }, function (err) {
                        reject(new Error(`
                            url: ${url}
                            http code: ${err.code}
                            error message: ${err.message}
                        `));
                    });
                });
            }, function (err) {
                reject(new Error(`
                    url: ${url}
                    http code: ${err.code}
                    error message: ${err.message}
                `));
            });
        });
    });
}