"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _config = require("../config");

var _config2 = _interopRequireDefault(_config);

var _rimraf = require("rimraf");

var _rimraf2 = _interopRequireDefault(_rimraf);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _nodePearls = require("node-pearls");

var _cliConsole = require("cli-console");

var _cliConsole2 = _interopRequireDefault(_cliConsole);

var _archiver = require("archiver");

var _archiver2 = _interopRequireDefault(_archiver);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

// 将js文件先打tar包再打zip包
function tarzip(file, outputFile, zipname) {
    return new Promise(function (resolve, reject) {
        var dir = _path2.default.dirname(file);

        var tarfile = _path2.default.join(dir, zipname + ".tar");

        var input = _fs2.default.createReadStream(file);
        var tarwrite = _fs2.default.createWriteStream(tarfile);
        // 打tar包
        let tar = (0, _archiver2.default)('tar');
        tar.on('error', function (err) {
            reject(err);
        });
        tar.append(input, {
            name: "framework.js"
        });
        tar.pipe(tarwrite);
        tar.finalize();
        tarwrite.on("close", function () {
            var tarread = _fs2.default.createReadStream(tarfile);
            var output = _fs2.default.createWriteStream(outputFile);
            // 打zip包
            let zip = (0, _archiver2.default)('zip');
            zip.on('error', function (err) {
                reject(err);
            });

            zip.append(tarread, {
                name: zipname + ".tar"
            });
            zip.pipe(output);

            zip.finalize();

            output.on("close", function () {
                _fs2.default.unlink(tarfile);
                resolve();
            });
        });
    });
}

exports.default = (() => {
    var _ref = _asyncToGenerator(function* (options) {
        // 要打包的平台
        const platforms = options.platforms ? options.platforms.split(",").map(function (platform) {
            return platform.trim();
        }) : [];
        const zipname = options.zipname || "framework";
        const iosZipname = options.iosZipname || zipname + "_package";
        const androidZipname = options.androidZipname || zipname + "_package";
        const pushBranch = options.pushBranch || "master";
        const versionRule = options.version;
        const currentDir = process.cwd();
        const tmpInstallDir = _path2.default.join(_config2.default["tmp-dir"], "framework-tmp-dir");

        _nodePearls.mkdirs.sync(tmpInstallDir);
        _fs2.default.writeFileSync(_path2.default.join(tmpInstallDir, "package.json"), JSON.stringify({
            name: "framework-tmp-dir",
            version: "0.0.1"
        }));
        yield Promise.all(platforms.map(function (platform) {
            return _asyncToGenerator(function* () {
                let modName = `enjoy-bundle-framework-${platform}`;
                yield (0, _nodePearls.runCommands)(`npm i ${modName}` + (versionRule ? `@${versionRule}` : ""), {
                    cwd: tmpInstallDir
                });
                let version = _nodePearls.readJson.sync(_path2.default.join(tmpInstallDir, "node_modules", modName, "package.json")).version;

                if (platform === "ios") {
                    let gitname = `elongreactnativepackages@${pushBranch}`;
                    let gitdir = _path2.default.join(_config2.default["tmp-dir"], gitname);

                    _cliConsole2.default.title("拉取项目");
                    if (!_fs2.default.existsSync(gitdir)) {
                        yield (0, _nodePearls.runCommands)(`
                    git clone git@git.17usoft.com:ios/elongreactnativepackages.git ${gitdir} -b ${pushBranch} --depth 1
                `);
                    } else {
                        yield (0, _nodePearls.runCommands)(`
                    git pull
                `, {
                            cwd: gitdir
                        });
                    }

                    _cliConsole2.default.title("压缩拷贝文件");
                    yield tarzip(_path2.default.join(_path2.default.join(tmpInstallDir, "node_modules", modName, options.release ? "index.js" : "index.dev.js")), _path2.default.join(gitdir, iosZipname + ".zip"), iosZipname);

                    _cliConsole2.default.title("提交git");
                    yield (0, _nodePearls.runCommands)(`
                git add .
                git commit -m 0:framework@${version}:true
                git push
            `, {
                        cwd: gitdir
                    });
                } else if (platform === "android") {
                    let gitname = `as_elongrnplugin_44_jsbundle@${pushBranch}`;
                    let gitdir = _path2.default.join(_config2.default["tmp-dir"], gitname);

                    _cliConsole2.default.title("拉取项目");
                    if (!_fs2.default.existsSync(gitdir)) {
                        yield (0, _nodePearls.runCommands)(`
                    git clone git@git.17usoft.com:elongtravel_android/as_elongrnplugin_44_jsbundle.git ${gitdir} -b ${pushBranch} --depth 1
                `);
                    } else {
                        yield (0, _nodePearls.runCommands)(`
                    git pull
                `, {
                            cwd: gitdir
                        });
                    }

                    _cliConsole2.default.title("压缩拷贝文件");
                    yield tarzip(_path2.default.join(_path2.default.join(tmpInstallDir, "node_modules", modName, options.release ? "index.js" : "index.dev.js")), _path2.default.join(gitdir, androidZipname + ".zip"), androidZipname);

                    _cliConsole2.default.title("提交git");
                    yield (0, _nodePearls.runCommands)(`
                git add .
                git commit -m 0:framework@${version}:true
                git push
            `, {
                        cwd: gitdir
                    });
                } else {}
            })();
        }));
    });

    return function (_x) {
        return _ref.apply(this, arguments);
    };
})();