"use strict";

var path = require("path");
var fs = require("fs");
var bundle = require("enjoy-bundle");
var uglifyJs = require("uglify-js");

bundle({
    root: __dirname,
    entries: ["index.js"],
    output: path.join(__dirname, "bundle"),
    loaders: [{
        test: /\.js$/,
        loader: function (content) {
            return content;
        }
    }],
    plugins: [function () {
        // 修改内部模块包装方式
        this.plugin("before-render-inner-code", function (info) {
            return info.content;
        });

        // 修改单独文件包装方式
        this.plugin("before-render-code", function (info) {
            if (info.file === "booter.js") {
                return info.codes.join("");
            } else {
                return info.codes.join("") + `;define("raw", document.currentScript.src);`;
            }
        });

        // 压缩
        this.plugin("before-write-bundle", function (info) {
            var code = info.code;
            try {
                code = uglifyJs.minify(info.code, {
                    fromString: true,
                    sourceMapInline: process.env.SM,
                    inSourceMap: process.env.SM ? "inline" : void 0
                }).code;
            } catch (e) {
                console.error(`文件${info.file}压缩失败`);
                console.log(e);
            }
            return code;
        });
    }]
}).then(function () {
    console.log("打包完成");
});