"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = request;

var _http = require("http");

var _http2 = _interopRequireDefault(_http);

var _url = require("url");

var _url2 = _interopRequireDefault(_url);

var _querystring = require("querystring");

var _querystring2 = _interopRequireDefault(_querystring);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function request(url, options) {
    url = _url2.default.parse(url);
    const method = options && options.method === "POST" ? "POST" : "GET";
    return new Promise(function (resolve, reject) {
        const req = _http2.default.request({
            hostname: url.hostname,
            port: url.port,
            path: url.path,
            method: method
        }, function (res) {
            const statusCode = res.statusCode;
            const contentType = res.headers['content-type'];

            let error;
            if (statusCode < 200 || statusCode >= 300) {
                reject({
                    type: "status",
                    value: statusCode
                });
                res.resume();
                return;
            } else if (!/^application\/json/.test(contentType)) {
                reject({
                    type: "content-type",
                    value: contentType
                });
                res.resume();
                return;
            }

            res.setEncoding('utf8');
            let rawData = '';
            res.on('data', chunk => rawData += chunk);
            res.on('end', () => {
                try {
                    let parsedData = JSON.parse(rawData);
                    resolve({
                        code: statusCode,
                        value: parsedData
                    });
                } catch (e) {
                    reject({
                        type: "parse-json",
                        value: e
                    });
                }
            });
        }).on('error', function (e) {
            reject({
                type: "other",
                value: e
            });
        });

        if (method === "POST") {
            req.write(_querystring2.default.stringify(options.data));
        }

        req.end();
    });
}

request.get = function (url) {
    return request(url);
};

request.post = function (url, data) {
    return request(url, {
        method: "POST",
        data: data
    });
};