import Http from "http";
import Url from "url";
import QueryString from "querystring";

export default function request(url, options){
    url = Url.parse(url);
    const method = options && options.method === "POST" ? "POST" : "GET";
    return new Promise(function(resolve, reject){
        const req = Http.request({
            hostname: url.hostname,
            port: url.port,
            path: url.path,
            method: method
        }, function(res) {
            const statusCode = res.statusCode;
            const contentType = res.headers['content-type'];

            let error;
            if (statusCode < 200 || statusCode >= 300) {
                reject({
                    type: "status",
                    value: statusCode
                });
                res.resume();
                return;
            } else if (!/^application\/json/.test(contentType)) {
                reject({
                    type: "content-type",
                    value: contentType
                });
                res.resume();
                return;
            }

            res.setEncoding('utf8');
            let rawData = '';
            res.on('data', (chunk) => rawData += chunk);
            res.on('end', () => {
                try {
                    let parsedData = JSON.parse(rawData);
                    resolve({
                        code: statusCode,
                        value: parsedData
                    });
                } catch (e) {
                    reject({
                        type: "parse-json",
                        value: e
                    });
                }
            });
        }).on('error', function(e) { 
            reject({
                type: "other",
                value: e
            });
        });

        if(method === "POST"){
            req.write(QueryString.stringify(options.data));
        }

        req.end();
    });
}

request.get = function(url){
    return request(url);
}

request.post = function(url, data){
    return request(url, {
        method: "POST",
        data: data
    });
}