import fs from "fs";
import crypto from "crypto";
import utils from "node-pearls";
import process from "child_process";

var exec = process.exec;

const AOSPATH = "aos";

var aosPlatform = 0;
var ptNum = "";

var PLATFORMNUM = { h5 : 0, weixin : 0, qq : 0, hybrid : 0, android : 0, ios : 0 };
var fileNum = Object.assign({},PLATFORMNUM);
var copyNum = Object.assign({},PLATFORMNUM);
var md5Num = Object.assign({},PLATFORMNUM);
/**
 * 删除文件夹,支持忽略文件
 * @param  {string} path   删除文件夹路径
 * @param  {string} expect 忽略文件路径
 */
function deletefolder(path,expect) {
	if(!fs.existsSync(path)){
		return false;
	}
	var list = fs.readdirSync(path);
	list.forEach(function(file) {
		if (fs.statSync(path + '/' + file).isDirectory()) {
			if(file !== expect){
				deletefolder(path + '/' + file);
				fs.rmdirSync(path + '/' + file);
			}
		} else {
			if(file !== expect){
				fs.unlinkSync(path + '/' + file);
			}
		}

	});
}
/**
 * 计算目录下所有文件的个数，累加平台下文件个数
 * @param  {String} path 目录路径
 * @param  {string} item 平台
 */
function countFile(path,item){
	var files = fs.readdirSync(path);//需要用到同步读取
	files.forEach(walk);
	function walk(file){
		var states = fs.statSync(path+'/'+file);         
		if(states.isDirectory()){
			countFile(path+'/'+file,item);
		}else{
			fileNum[item]++;
		}     
	}
}

/**
 * copy file
 * @param  {string} file 文件名
 * @param  {string} src  文件初始路径
 * @param  {string} dest 文件目的路径
 * @param  {string} item 平台
 */
function copyfile(file, src, dest, item) {
	return new Promise((resolve, reject) => {
		var newdest = file.replace(src, dest);
		var readStream = fs.createReadStream(file,{encoding:'binary',bufferSize:11});
		var writeStream = fs.createWriteStream(newdest,{ encoding: "binary" });
		readStream.pipe(writeStream);
		readStream.on('end', function () {
			resolve(true);
		});

		readStream.on('error', function () {
			console.log('copy error');
			reject(false);
		});
	})
}

/**
 * 遍历处理打包路径
 * @param  {string} path 遍历路径
 * @param  {string} src  copy初始路径
 * @param  {string} dest copy目的路径
 * @param  {string} item 平台
 */
function walk(path, src, dest, item){
	return new Promise((resolve,reject)=>{
		var map = fs.readdir(path,(err,files)=>{
			if(err){
				reject();
				return;
			}else{
				files.forEach( async function(file,index) {
					
					if (!fs.statSync(path + '/' + file).isDirectory()) {
						try{
							var result = await copyfile(src + '/' + file, src, dest);//.then(async()=>{
							
							if(result){
								copyNum[item]++;
							
								fs.stat(src + '/' + file, function(err,info){
									console.log(file+":============"+info.size);
									if(copyNum[item] == fileNum[item]){
										aosPlatform++;
										if(ptNum == aosPlatform){
											resolve();
										}
									}
								})
							}
						}catch(ex){
							reject(ex);
						}
					}else{
						var subpath = path + '/' + file;
						var newdest = subpath.replace(src, dest);
						if (!fs.existsSync(newdest))
							fs.mkdirSync(newdest);
						try{
							var result = await  walk(path + '/' + file, subpath, newdest, item);
							if(copyNum[item] == fileNum[item]){
								aosPlatform++;
								if(ptNum == aosPlatform){
									resolve();
								}
							}
						}catch(ex){
							console.log(ex);
							reject(ex);
						}
					}
				})
			}
		});
	})
	
}

/**
 * 整包压缩 + md5
 * @param  {string} item 平台
 */
function compress(item) {

	var filename = "/MD5.list";
	var zipPath = AOSPATH + "/output";
	
	if (!fs.existsSync(zipPath))
		fs.mkdirSync(zipPath);
	var cmd = "zip -r ./output.zip ./";
	console.log(cmd);
	
	return new Promise((resolve,reject)=>{
		exec( cmd, {cwd: './'+zipPath }, function(err, stdout , stderr ) {
			if(err){
				reject(err);
				return;
			}else{
				var content = fs.readFileSync(zipPath + "/output.zip");
				var md5 = crypto.createHash('md5');
				md5.update(content);
				md5 = md5.digest('hex');
				fs.writeFile(AOSPATH + filename, md5 + "	./output.zip", "utf-8",function(err){
					if(err){
						reject(err);
					}else{
						deletefolder(zipPath,'output.zip');
						resolve(true);
					}
				});
			}
		});
	})
}

/**
 * 拆包md5
 * @param  {string} path 加密文件存储目录
 * @param  {string} item 平台
 */
function createMd5(path,item){
	return new Promise((resolve,reject)=>{
		fs.readdir(path,(err,files)=>{
			if(err){
				reject();
				return;
			}else{
				files.forEach( function(file,index) {
					//短名称去掉了目录
					if (!fs.statSync(path + '/' + file).isDirectory()){
						function handleResult(err){
							if(err){
								reject(err);
							}
							md5Num[item]++;
							if(fileNum[item] == md5Num[item]){
								resolve(true);
							}
						}
						// fs.readFile(path + '/' + file,"utf-8",(err,content)=>{
						// 	var md5 = crypto.createHash('md5');
						// 	md5.update(content);
						// 	md5 = md5.digest('hex');
						// 	console.log("file md5:"+file+" "+ md5);
						// 	if (!fs.existsSync(AOSPATH + "/MD5.list")){
						// 		fs.writeFile(AOSPATH + "/MD5.list", md5 + "	./"+file, "utf-8", handleResult);
						// 	}else{
						// 		fs.appendFile(AOSPATH + "/MD5.list", "\r\n"+md5 + "	./"+file, "utf-8", handleResult);
						// 	}
						// });

						exec( "md5sum ./"+file, {cwd: './'+path }, function(err, stdout , stderr ) {
							if(err){
								reject(err);
								return;
							}else{
								console.log("file md5"+stdout.substring(0,32));
								if (!fs.existsSync(AOSPATH + "/MD5.list")){
									fs.writeFile(AOSPATH + "/MD5.list", stdout.substring(0,32)+"	./"+file, "utf-8", handleResult);
								}else{
									fs.appendFile(AOSPATH + "/MD5.list", "\r\n"+stdout.substring(0,32)+"	./"+file, "utf-8", handleResult);
								}
							}
						})
					}
				})
			}
		})
	})
}

export default function(options){
	var src = "";
	ptNum = options.platform.length;
	var version = options.version;

	deletefolder(AOSPATH);
	return new Promise((resolve,reject)=>{
		options.platform.map(async function(item,index){
			src = "web/" + item + "/bundle";

			var dest ="";
			if(!!options.combo || !!options.c){
				src = "web/" + item + "/bundle/combo";
				dest = "aos/output/"+item+"/"+version;
			}else{
				src = "web/" + item + "/bundle/aos";
				dest = "aos/output";
			}

			countFile(src,item);
			console.log(item+"平台： "+ fileNum[item]);
			
			var path = dest.replace(version, "");

			if (!fs.existsSync(dest)){
				var dirs = dest.split("/");
				var curPath = "";
				for (var i = 0; i < dirs.length; i++) {
					curPath += dirs[i] + "/";

					if (!fs.existsSync(curPath))
						fs.mkdirSync(curPath);
				}
			}

			walk(src, src, dest, item).then( async ()=>{
				try{
					if(!!options.combo || !!options.c){
						var result = await compress(item);
						if (result){
							resolve(true);
						}
					}else{
						var result = await createMd5(dest,item);
						if(result){
							resolve(true);
						}
					}
				}catch(ex){
					console.log(ex);
					reject(ex);
				}
			}).catch((ex)=>{
				console.log(ex);
				reject(ex);
			})
		})
	})
}